"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = useCreateReducerWithEffect;
exports.SideEffect = exports.UpdateWithSideEffect = exports.NoUpdate = exports.Update = void 0;

var _react = require("react");

const NO_UPDATE_SYMBOL = Symbol("NO_UPDATE_SYMBOL");
const REMOVE_EXECUTED_SIDE_EFFECT = Symbol("REMOVE_EXECUTED_SIDE_EFFECT");

const Update = newState => ({
  newState
});

exports.Update = Update;

const NoUpdate = () => NO_UPDATE_SYMBOL;

exports.NoUpdate = NoUpdate;

const UpdateWithSideEffect = (newState, newSideEffect) => ({
  newState,
  newSideEffect: {
    effect: newSideEffect,
    id: Math.random() * 10000
  }
});

exports.UpdateWithSideEffect = UpdateWithSideEffect;

const SideEffect = newSideEffect => ({
  newSideEffect: {
    effect: newSideEffect,
    id: Math.random() * 10000
  }
});

exports.SideEffect = SideEffect;

async function executeSideEffects({
  sideEffects,
  state,
  dispatch
}) {
  await Promise.all(sideEffects.forEach(sideEffect => {
    sideEffect.effect(state, dispatch);
    dispatch({
      type: REMOVE_EXECUTED_SIDE_EFFECT,
      sideEffect
    });
  }));
}

function finalReducer(reducer) {
  return function ({
    state,
    sideEffects = []
  }, action) {
    let {
      newState,
      newSideEffect
    } = reducer(state, action);
    let newSideEffects = sideEffects;
    newSideEffects = newSideEffect ? [...newSideEffects, newSideEffect] : newSideEffects;

    if (action.type === REMOVE_EXECUTED_SIDE_EFFECT) {
      newSideEffects = sideEffects.filter(effect => effect.id !== action.sideEffect.id);
    } else if (action === NO_UPDATE_SYMBOL) {
      return {
        state,
        sideEffects
      };
    }

    return {
      state: newState || state,
      sideEffects: newSideEffects
    };
  };
}

function useCreateReducerWithEffect(reducer, initialState) {
  const [{
    state,
    sideEffects
  }, dispatch] = (0, _react.useReducer)(finalReducer(reducer), {
    state: initialState,
    sideEffects: []
  });
  (0, _react.useEffect)(() => {
    if (sideEffects.length) {
      async function runSideEffects() {
        await executeSideEffects({
          sideEffects,
          state,
          dispatch
        });
      }

      runSideEffects();
    }
  }, [sideEffects]); //eslint-disable-line

  return [state, dispatch];
}